CREATE PROCEDURE [dbo].[Custom_CollectDTUDataForDevice]
    -- You can enter the DeviceID if you want to only extract data 
    -- for a specific device or leave it as -1 to do everything
    @DeviceID smallint = -1,

    -- The number of days to get data for, as well as the number 
	-- of days to purge from [dbo].[AzureDatabaseDTUdata] table
    @DaysPurged smallint = 14,

    -- These define the CounterIDs in case they ever change. 
    @ProcessorCounterID     smallint = 1858, -- Processor (Default)
    @DiskReadCounterID      smallint = 64,   -- Disk Read/Sec (DiskCounter)
    @DiskWritesCounterID    smallint = 67,   -- Disk Writes/Sec (Diskcounter)
    @LogBytesFlushCounterID smallint = 166,  -- Log Bytes Flushed/Sec (DatabaseCounter)

	@debug bit = 0
AS 
BEGIN
  SET NOCOUNT ON;

  DECLARE @BeginDate        datetime = DATEADD(DAY,-@DaysPurged,GETDATE()),
		  @ConvertBeginDate bigint = DATEDIFF(SECOND,36524,DATEADD(DAY,-@DaysPurged,GETDATE()))/5,
          @RollupLevelID    smallint,             -- 36524 = '2000-01-01' = 365 years + 24 leap days
          @MaxDatetime      int,
		  @RangeSizeMinutes int = 0,
          @SQLBase          nvarchar(max) = N'SELECT PD.DeviceID, 
     CONVERT(smalldatetime,DATEADD(SECOND,PD.[Timestamp]*5,36524)) AS [TimeStamp],
     PC.[CounterName],
     MAX(PD.Value) AS Value,
     ''_Total'' AS InstanceName
   FROM dbo.PerformanceAnalysisData AS PD WITH (NOLOCK)
   LEFT JOIN dbo.PerformanceAnalysisCounter AS PC WITH (NOLOCK)
     ON PC.ID = PD.PerformanceAnalysisCounterID
   WHERE PD.InstanceName = ''_Total''
    ' + CASE WHEN @DeviceID <> -1 THEN N' AND PD.DeviceID = @DeviceID' ELSE N'' END + N'
  	 AND PD.PerformanceAnalysisCounterID = @CounterID
  	 AND PD.[Timestamp] > @MaxDatetime
  	GROUP BY PD.DeviceID, CONVERT(smalldatetime,DATEADD(SECOND,PD.[Timestamp]*5,36524)),PC.[CounterName]',
          @SQLStmt          nvarchar(max),
          @SQLParams        nvarchar(1000) = N'@DeviceID smallint,@CounterID smallint,@MaxDatetime int',
          @RollupSuffix     nvarchar(128);

  IF NOT EXISTS (SELECT 1 FROM sys.tables WHERE name = 'AzureDatabaseDTUdata')
  BEGIN
    CREATE TABLE dbo.AzureDatabaseDTUdata
    (
	  ID bigint identity(1,1) not null,
      DeviceID smallint not null,
      [TimeStamp] datetime not null,
      CounterName nvarchar(256) not null,
      [Value] float not null,
      InstanceName nvarchar(256) not null,
      CONSTRAINT PK_AzureDatabaseDTUdata PRIMARY KEY (ID)
	);
  END


  /* processor */


  SET @MaxDateTime = NULL;

  -- Find most recent timestamp added  
  SELECT @MaxDateTime = DATEDIFF(SECOND,36524,COALESCE(MAX([TimeStamp]),36524))/5
    FROM dbo.AzureDatabaseDTUdata
    WHERE CounterName = N'% Processor Time';

   IF @MaxDateTime IS NOT NULL
   BEGIN
     SET @RollupLevelID = dbo.fnRollupLevelForRangeSizeAndCounterInMinutes
         (@RangeSizeMinutes, @ConvertBeginDate, @ProcessorCounterID);
  
     SET @RollupSuffix = CASE @RollupLevelID
	   WHEN 0 THEN dbo.fnGetDataPartitionSuffixForCounter(@ProcessorCounterID)
       ELSE N'Rollup' + CAST(@RollupLevelID AS varchar(2)) END;

     SET @SQLStmt = REPLACE(@SQLBase, N'AnalysisData', N'AnalysisData' + @RollupSuffix);

     IF @debug = 1 PRINT @MaxDateTime;
	   
     INSERT INTO [dbo].[AzureDatabaseDTUdata](DeviceID,[TimeStamp],[CounterName],[Value],[InstanceName])
     EXEC sys.sp_executesql @SQLStmt, @SQLParams,
          @DeviceID    = @DeviceID,
          @CounterID   = @ProcessorCounterID,
          @MaxDatetime = @MaxDatetime;
   END

  /*  disk reads  */


  SET @MaxDateTime = NULL;

  -- Find most recent timestamp added  
  SELECT @MaxDateTime = DATEDIFF(SECOND,36524,COALESCE(MAX([TimeStamp]),36524))/5
    FROM dbo.AzureDatabaseDTUdata
    WHERE CounterName = N'Disk Reads/sec';
  
   IF @MaxDateTime IS NOT NULL
   BEGIN
     SET @RollupLevelID = dbo.fnRollupLevelForRangeSizeAndCounterInMinutes
         (@RangeSizeMinutes, @ConvertBeginDate, @DiskReadCounterID);
  
     SET @RollupSuffix = CASE @RollupLevelID
	   WHEN 0 THEN dbo.fnGetDataPartitionSuffixForCounter(@DiskReadCounterID)
       ELSE N'Rollup' + CAST(@RollupLevelID AS varchar(2)) END;

     SET @SQLStmt = REPLACE(@SQLBase, N'AnalysisData', N'AnalysisData' + @RollupSuffix);

     IF @debug = 1 PRINT @SQLStmt;
	   
     INSERT INTO [dbo].[AzureDatabaseDTUdata](DeviceID,[TimeStamp],[CounterName],[Value],[InstanceName])
     EXEC sys.sp_executesql @SQLStmt, @SQLParams,
          @DeviceID    = @DeviceID,
          @CounterID   = @DiskReadCounterID,
          @MaxDatetime = @MaxDatetime;
   END  
  
  /*  disk writes  */

  SET @MaxDateTime = NULL;

  -- Find most recent timestamp added  
  SELECT @MaxDateTime = DATEDIFF(SECOND,36524,COALESCE(MAX([TimeStamp]),36524))/5
    FROM dbo.AzureDatabaseDTUdata
    WHERE CounterName = N'Disk Writes/sec';
  
   IF @MaxDateTime IS NOT NULL
   BEGIN
     SET @RollupLevelID = dbo.fnRollupLevelForRangeSizeAndCounterInMinutes
         (@RangeSizeMinutes, @ConvertBeginDate, @DiskWritesCounterID);
  
     SET @RollupSuffix = CASE @RollupLevelID
	   WHEN 0 THEN dbo.fnGetDataPartitionSuffixForCounter(@DiskWritesCounterID)
       ELSE N'Rollup' + CAST(@RollupLevelID AS varchar(2)) END;

     SET @SQLStmt = REPLACE(@SQLBase, N'AnalysisData', N'AnalysisData' + @RollupSuffix);

     IF @debug = 1 PRINT @SQLStmt;
	   
     INSERT INTO [dbo].[AzureDatabaseDTUdata](DeviceID,[TimeStamp],[CounterName],[Value],[InstanceName])
     EXEC sys.sp_executesql @SQLStmt, @SQLParams,
          @DeviceID    = @DeviceID,
          @CounterID   = @DiskWritesCounterID,
          @MaxDatetime = @MaxDatetime;
   END  

  /*  log bytes  */
  
  SET @MaxDateTime = NULL;

  -- Find most recent timestamp added  
  SELECT @MaxDateTime = DATEDIFF(SECOND,36524,COALESCE(MAX([TimeStamp]),36524))/5
    FROM dbo.AzureDatabaseDTUdata
    WHERE CounterName = N'Log Bytes Flushed/sec';
  
   IF @MaxDateTime IS NOT NULL
   BEGIN
     SET @RollupLevelID = dbo.fnRollupLevelForRangeSizeAndCounterInMinutes
         (@RangeSizeMinutes, @ConvertBeginDate, @LogBytesFlushCounterID);
  
     SET @RollupSuffix = CASE @RollupLevelID
	   WHEN 0 THEN dbo.fnGetDataPartitionSuffixForCounter(@LogBytesFlushCounterID)
       ELSE N'Rollup' + CAST(@RollupLevelID AS varchar(2)) END;

     SET @SQLStmt = REPLACE(@SQLBase, N'AnalysisData', N'AnalysisData' + @RollupSuffix);

     IF @debug = 1 PRINT @SQLStmt;
	   
     INSERT INTO [dbo].[AzureDatabaseDTUdata](DeviceID,[TimeStamp],[CounterName],[Value],[InstanceName])
     EXEC sys.sp_executesql @SQLStmt, @SQLParams,
          @DeviceID    = @DeviceID,
          @CounterID   = @LogBytesFlushCounterID,
          @MaxDatetime = @MaxDatetime;
   END
  
  /*  Clean up old data  */
  
  -- There is the possibility due to your configured cleanup purge schedule on SentryOne
  -- that you could be missing counter information for some dates. Understand there could be
  -- issues if you are reading data from dates before you started offloading
  
  -- Deletes data based on what you set as the purge period
  DELETE [dbo].[AzureDatabaseDTUdata]
  WHERE [TimeStamp] < DATEADD(DAY,-@DaysPurged,GETDATE());
END
GO